VERSION 5.00
Begin VB.UserControl TrayIcon 
   BorderStyle     =   1  'Fixed Single
   CanGetFocus     =   0   'False
   ClientHeight    =   1140
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   2730
   ClipBehavior    =   0  'None
   ClipControls    =   0   'False
   EditAtDesignTime=   -1  'True
   FontTransparent =   0   'False
   HasDC           =   0   'False
   HitBehavior     =   0  'None
   InvisibleAtRuntime=   -1  'True
   PaletteMode     =   4  'None
   Picture         =   "TrayIcon.ctx":0000
   ScaleHeight     =   1140
   ScaleWidth      =   2730
   ToolboxBitmap   =   "TrayIcon.ctx":0442
   Begin VB.Image imgIcon 
      Height          =   480
      Left            =   600
      Picture         =   "TrayIcon.ctx":0754
      Top             =   360
      Visible         =   0   'False
      Width           =   480
   End
End
Attribute VB_Name = "TrayIcon"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'==================================================================
'
' Tray Icon Control
'
' 30 September 2001
'
' You're free in using this software, as long as the
' orignal copyright/author information remains intact.
' In other words, respecting the person(s) who made it
' possible for you to view/learn something of this software.
'
' Copyright  2001 - Diederik van der Boor
' http://www.codingdomain.com
'
'==================================================================


'------------------------------------------------------------------
' Current Variables

Private TrayIconData As NOTIFYICONDATA
Private CurVisible As Boolean
Private CurIcon As New StdPicture

'------------------------------------------------------------------
' Tray messages will be received by the control

' Using this message; which results in a UserControl_MouseUp event
' All the parameters are perfectly parsed by VB, and the event is fired
Private Const WM_RBUTTONUP = &H205

Private Enum TrayEvents
  TRAY_MSG_MOUSEMOVE = 7680
  TRAY_MSG_LEFTBTN_DOWN = 7695
  TRAY_MSG_LEFTBTN_UP = 7710
  TRAY_MSG_LEFTBTN_DBLCLICK = 7725
  TRAY_MSG_RIGHTBTN_DOWN = 7740
  TRAY_MSG_RIGHTBTN_UP = 7755
  TRAY_MSG_RIGHTBTN_DBLCLICK = 7770
End Enum

' Tray Flags
Private Enum TrayMessages
  NIM_ADD = &H0
  NIM_MODIFY = &H1
  NIM_DELETE = &H2
End Enum

Private Enum TrayStructFlags
  NIF_MESSAGE = &H1
  NIF_ICON = &H2
  NIF_TIP = &H4
End Enum

Private Type NOTIFYICONDATA
  cbSize As Long
  hWnd As Long
  uID As Long
  uFlags As Long
  uCallbackMessage As Long
  hIcon As Long
  szTip As String * 64  ' Max Text length: 63
End Type

Private Declare Function Shell_NotifyIcon Lib "shell32.dll" Alias "Shell_NotifyIconA" (ByVal dwMessage As TrayMessages, lpData As NOTIFYICONDATA) As Long


'------------------------------------------------------------------
' Events from Tray Icon

Public Event LeftMouseDown()
Public Event LeftMouseDblClick()
Public Event LeftMouseUp()
Public Event RightMouseDown()
Public Event RightMouseDblClick()
Public Event RightMouseUp()
Public Event MouseMove()


'------------------------------------------------------------------
' Initialize

Private Sub UserControl_Initialize()
  TrayIconData.cbSize = LenB(TrayIconData)
  TrayIconData.uID = ObjPtr(Me)
  TrayIconData.hWnd = UserControl.hWnd
End Sub

Private Sub UserControl_InitProperties()
  ' Done with first initialization
  If UserControl.Ambient.UserMode Then
    TrayIconData.uCallbackMessage = WM_RBUTTONUP
    TrayIconData.uFlags = NIF_MESSAGE
  End If
  Set Me.Icon = imgIcon.Picture
  Me.ToolTip = "Tray Icon Sample"
End Sub


'------------------------------------------------------------------
' Property Bag

Private Sub UserControl_ReadProperties(PropBag As PropertyBag)
  ' Done with second initialization
  If UserControl.Ambient.UserMode Then
    TrayIconData.uCallbackMessage = WM_RBUTTONUP
    TrayIconData.uFlags = NIF_MESSAGE
  End If
  Set Me.Icon = PropBag.ReadProperty("Icon", imgIcon.Picture)
  Me.ToolTip = PropBag.ReadProperty("ToolTip", "Tray Icon Control")
  Me.Visible = PropBag.ReadProperty("Visible", False)
End Sub

Private Sub UserControl_WriteProperties(PropBag As PropertyBag)
  Call PropBag.WriteProperty("Icon", Me.Icon, imgIcon.Picture)
  Call PropBag.WriteProperty("ToolTip", Me.ToolTip, "Tray Icon Control")
  Call PropBag.WriteProperty("Visible", Me.Visible)
End Sub


'------------------------------------------------------------------
' Keep fixed. COL/OLE has some beautiful implementable classes for
' that, but that's almost beyond our limits in VB (keep it simple!_

Private Sub UserControl_Resize()
  Static Resizing As Boolean
  If Resizing Then Exit Sub ' Caused by this event handler
  Resizing = True
  UserControl.Width = 540 '36 * 15
  UserControl.Height = 540
  Resizing = False
End Sub


'------------------------------------------------------------------
' This event is called automatically when the tray fires events.
' The tray has been ordered to send information within
' this particular event.

Private Sub UserControl_MouseUp(Button As Integer, Shift As Integer, X As Single, Y As Single)
  ' Y = 0; X = Event, Button is right button
  If Button = vbRightButton And Shift = 0 And Y = 0 Then
    Select Case X
      Case TRAY_MSG_LEFTBTN_DOWN:      RaiseEvent LeftMouseDown
      Case TRAY_MSG_LEFTBTN_UP:        RaiseEvent LeftMouseUp
      Case TRAY_MSG_LEFTBTN_DBLCLICK:  RaiseEvent LeftMouseDblClick
      Case TRAY_MSG_RIGHTBTN_DOWN:     RaiseEvent RightMouseDown
      Case TRAY_MSG_RIGHTBTN_UP:       RaiseEvent RightMouseUp
      Case TRAY_MSG_RIGHTBTN_DBLCLICK: RaiseEvent RightMouseDblClick
      Case TRAY_MSG_MOUSEMOVE:         RaiseEvent MouseMove
      Case Else: Debug.Print "TrayIcon Error: Unknown event " & X
    End Select
  End If
End Sub


'------------------------------------------------------------------
' Load or unload the icon

Private Sub UserControl_Terminate()
  Me.Visible = False
End Sub

Public Property Get Visible() As Boolean
  Visible = CurVisible
End Property

Public Property Let Visible(NewValue As Boolean)
  If CurVisible Then
    If Shell_NotifyIcon(NIM_DELETE, TrayIconData) = 0 Then Err.Raise 12345, "TrayIcon.Visible", "Could not remove tray icon!"
  End If
  If NewValue Then
    TrayIconData.uFlags = NIF_MESSAGE Or NIF_TIP Or IIf(TrayIconData.hIcon <> 0, NIF_ICON, 0)
    If Shell_NotifyIcon(NIM_ADD, TrayIconData) = 0 Then Err.Raise 12345, "TrayIcon.Visible", "Could not add tray icon!"
  End If
  CurVisible = NewValue
  PropertyChanged "Visible"
End Property


'------------------------------------------------------------------
' Tooltip

Public Property Get ToolTip() As String
  Dim Pos As Long
  Pos = InStr(TrayIconData.szTip, Chr(0))
  If Pos Then ToolTip = Left(TrayIconData.szTip, Pos - 1)
End Property

Public Property Let ToolTip(NewValue As String)
  If (Len(NewValue) > 63) Or (InStr(1, NewValue, Chr(0)) > 0) Then Error 380
  TrayIconData.szTip = NewValue & Chr(0)
  If CurVisible Then
    TrayIconData.uFlags = NIF_TIP
    If Shell_NotifyIcon(NIM_MODIFY, TrayIconData) = 0 Then Err.Raise 12345, "TrayIcon.ToolTip", "Could not update tray icon!"
  End If
  PropertyChanged "ToolTip"
End Property


'------------------------------------------------------------------
' The Icon

Public Property Get Icon() As StdPicture
  If CurIcon.Handle <> TrayIconData.hIcon Then Err.Raise 5, , "Internal Error in TrayIcon control"
  Set Icon = CurIcon
End Property

Public Property Let Icon(NewValue As StdPicture)
  Set Icon = NewValue
End Property

Public Property Set Icon(NewValue As StdPicture)
  If Not NewValue Is Nothing Then
    If NewValue.Handle <> 0 Then
      If NewValue.Type <> vbPicTypeIcon Then Err.Raise 380, , "A real icon object is requierd"
    End If
    Set CurIcon = NewValue
    TrayIconData.hIcon = CurIcon.Handle
  Else
    Set CurIcon = New StdPicture
    TrayIconData.hIcon = 0
  End If
  If CurVisible Then
    TrayIconData.uFlags = NIF_ICON
    If Shell_NotifyIcon(NIM_MODIFY, TrayIconData) = 0 Then Err.Raise 12345, "TrayIcon.hIcon", "Could not update tray icon!"
  End If
  PropertyChanged "Icon"
End Property

